#!/bin/bash
# Define the location of your check_nrpe binary
check_nrpe=/usr/local/nagios/libexec/check_nrpe
#
# Define the location of your check_nt binary
check_nt=/usr/local/nagios/libexec/check_nt
#
# ------------------------------------------------------------------------------
# box293_drive_inventory_windows - Report on all hard disk drives in Windows
# Copyright (C) 2016 Troy Lea AKA Box293
#
# This plugin will report what hard drives in the range A: - Z: exist on a
# Windows machine. This plugin uses NSClient++ to gather the information.
# The plugin will use check_nrpe (by default) or check_nt (if specified)
# to query the Windows machine. This plugin was created to be able to get
# a list of drives in Windows without requiring an additional script to be
# deployed on the Windows server. As long as NSClient++ is configured to
# respond to performance counter checks then this plugin will work.
# The plugin simply queries the "\LogicalDisk(DRIVE LETTER)\Free Megabytes"
# performance counter object for all drive letters in the range A: - Z:.
# If it doesn't get a valid response back it assumes that disk does not exist.
# The plugin will return an OK state unless there is some sort of a problem
# and then it will simply return an UNKNOWN state.
# The check_nrpe and check_nt binary locations are defined at the top of this
# pluging, adjust them as required.
# NSClient++ NRPE config file will require the following options enabled:
# v0.3.x
# [NRPE]
# allow_arguments=1
# allow_nasty_meta_chars=1
#
# v0.4.x Onwards
# [/settings/NRPE/server]
# allow nasty characters = 1
# allow arguments = 1
#
# Usage:
# box293_drive_inventory_windows.sh [host] [type] [port] [secret]
#
# [host] is a required argument, the IP address or DNS name of the Windows
# machine you want to query.
#
# [type] is which type of performance counter check you want to query. By
# default it uses "check_pdh" which exists in NSClient++ 0.4.x onwards. If you
# have an older version of NSClient++ then you can use "CheckCounter" instead.
# If you don't have NRPE enabled on NSClient++ then you can force it to use
# "check_nt" however this method should be avoided.
#
# [port] allows you define the port used. Port 5666 is used for NRPE and port
# 12489 is user for check_nt.
#
# [secret] only applies to check_nt and if you want to use a secret you MUST
# also provide the port argument.
#
# Example 1:
# ./box293_drive_inventory_windows.sh 10.25.11.3
# Drives Detected: C: D: E:
#
# Example 2:
# box293_drive_inventory_windows.sh 10.25.14.2 CheckCounter 5666
# Drives Detected: C: E: F: G: H: Q:
#
# Example 3:
# box293_drive_inventory_windows.sh 10.25.14.51 COUNTER 12489 Str0ngPassw0rd
# Drives Detected: C:
#
#
# Created by: Troy Lea <plugins@box293.com> Twitter: @Box293
# See all my Nagios Projects on the Nagios Exchange: 
#	http://exchange.nagios.org/directory/Owner/Box293/1
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.
#
# ------------------------------------------------------------------------------
# 				Version Notes
# ------------------------------------------------------------------------------
# Version : 2016-10-23
# Date    : October 23 2016
# Notes   : This is the first official release of the plugin
#
# Version : 2016-12-09
# Date    : December 09 2016
# Notes   : Added improved error handling logic.
# ------------------------------------------------------------------------------

function print_help {
	echo "box293_drive_inventory_windows - Report on all hard disk drives in Windows"
	echo "Copyright (C) 2016 Troy Lea AKA Box293"
	echo ""
	echo "This plugin will report what hard drives in the range A: - Z: exist on a Windows machine. This plugin uses NSClient++ to gather the information."
	echo "The plugin will use check_nrpe (by default) or check_nt (if specified) to query the Windows machine. This plugin was created to be able to get a list of drives in Windows without requiring an additional script to be deployed on the Windows server. As long as NSClient++ is configured to respond to performance counter checks then this plugin will work."
	echo "The plugin simply queries the \"\LogicalDisk(DRIVE LETTER)\Free Megabytes\" performance counter object for all drive letters in the range A: - Z: and if it doesn't get a valid response back it assumes that disk does not exist."
	echo "The plugin will return an OK state unless there is some sort of a problem and then it will simply return an UNKNOWN state."
	echo "The check_nrpe and check_nt binary locations are defined at the top of this pluging, adjust them as required."
	echo "NSClient++ NRPE config file will require the following options enabled:"
	echo "v0.3.x"
	echo "[NRPE]"
	echo "allow_arguments=1"
	echo "allow_nasty_meta_chars=1"
	echo ""
	echo "v0.4.x Onwards"
	echo "[/settings/NRPE/server]"
	echo "allow nasty characters = 1"
	echo "allow arguments = 1"
	echo ""
	echo "Usage:"
	echo "box293_drive_inventory_windows.sh [host] [type] [port] [secret]"
	echo ""
	echo "[host] is a required argument, the IP address or DNS name of the Windows machine you want to query."
	echo ""
	echo "[type] is which type of performance counter check you want to query. By default it uses \"check_pdh\" which exists in NSClient++ 0.4.x onwards. If you have an older version of NSClient++ then you can use \"CheckCounter\" instead."
	echo "If you don't have NRPE enabled on NSClient++ then you can force it to use \"check_nt\" however this method should be avoided."
	echo ""
	echo "[port] allows you define the port used. Port 5666 is used for NRPE and port 12489 is user for check_nt."
	echo ""
	echo "[secret] only applies to check_nt and if you want to use a secret you MUST also provide the port argument."
	echo ""
	echo "Example 1:"
	echo "./box293_drive_inventory_windows.sh 10.25.11.3"
	echo "Drives Detected: C: D: E:"
	echo ""
	echo "Example 2:"
	echo "box293_drive_inventory_windows.sh 10.25.14.2 CheckCounter 5666"
	echo "Drives Detected: C: E: F: G: H: Q:"
	echo ""
	echo "Example 3:"
	echo "box293_drive_inventory_windows.sh 10.25.14.51 COUNTER 12489 Str0ngPassw0rd"
	echo "Drives Detected: C:"
}

if [ "$1" == '--help' ] ; then
	print_help
	exit 0
fi

# Determine if the host address was provided
if [ -z "$1" ] ; then
	echo "No host address supplied (this should be the 1st argument)"
	echo ""
	print_help
	exit 3
fi

# Determine which check method to use
if [ -z "$2" ] ; then
	check_command="check_pdh"
elif [ "$2" == "check_pdh" ] ; then
	check_command="check_pdh"
elif [ "$2" == "CheckCounter" ] ; then
	check_command="CheckCounter"
else
	if [ "$2" == "COUNTER" ] ; then
		check_command="COUNTER"
		if [ -z "$3" ] ; then
			check_nt_port="12489"
		else
			check_nt_port="$3"
		fi
		if [ -z "$4" ] ; then
			check_nt_secret="-s ''"
		else
			check_nt_secret="-s '$4'"
		fi
	test_command_to_execute="$check_nt -H $1 -p $check_nt_port $check_nt_secret -v CLIENTVERSION 2>&1"
	fi
fi

if [ "$check_command" != "COUNTER" ] ; then
	if [ -z "$3" ] ; then
			check_nrpe_port="5666"
		else
			check_nrpe_port="$3"
	fi
	test_command_to_execute="$check_nrpe -H $1 -p $check_nrpe_port 2>&1"
fi


# Test to make sure the agent is responding
test_command_output=$(eval $test_command_to_execute)
if [ $? -eq 0 ] ; then
	# Agent appears to be responding

	# Make sure that check_pdh works before proceeding
	if [ "$check_command" == "check_pdh" ] ; then
		test_check_pdh="$check_nrpe -H $1 -p $check_nrpe_port -c $check_command -a 'counter=\LogicalDisk(C:)\Free Megabytes' 2>&1"
		test_check_pdh_output=$(eval $test_check_pdh)
		test_check_pdh_exit_code=$?
		if [ $test_check_pdh_exit_code -ne 0 ] ; then
			if [[ $test_check_pdh_output == Failed* ]] ; then
				printf "NSClient++ Did not respond nicely: $test_check_pdh_output\n"
				exit 3
			else
				# Most likely check_pdh isn't going to work, try to CheckCounter
				test_checkcounter="$check_nrpe -H $1 -p $check_nrpe_port -c $check_command -a 'counter=\LogicalDisk(C:)\Free Megabytes' 2>&1"
				test_checkcounter_output=$(eval $test_check_pdh)
				test_checkcounter_exit_code=$?
				if [ $test_checkcounter_exit_code -ne 0 ] ; then
					if [[ $test_checkcounter_output == Failed* ]] ; then
						printf "NSClient++ Did not respond nicely: $test_checkcounter_output\n"
						exit 3
					fi
				else
					check_command="CheckCounter"
				fi
			fi
		fi
	fi

	# Start the exit message
	exit_message="Drives Detected:"

	for drive_letter in {A..Z} ; do
		#echo $drive_letter
		if [ "$check_command" == "COUNTER" ] ; then
			command_to_execute="$check_nt -H $1 -p $check_nt_port $check_nt_secret -v $check_command -l '\LogicalDisk($drive_letter:)\Free Megabytes' 2>&1"
		elif [ "$check_command" == "CheckCounter" ] ; then
			command_to_execute="$check_nrpe -H $1 -p $check_nrpe_port -c $check_command -a 'Counter=\LogicalDisk($drive_letter:)\Free Megabytes' 2>&1"
		else
			command_to_execute="$check_nrpe -H $1 -p $check_nrpe_port -c $check_command -a 'counter=\LogicalDisk($drive_letter:)\Free Megabytes' 2>&1"
		fi
		#echo $command_to_execute
		command_output=$(eval $command_to_execute)
		if [ $? -eq 0 ] ; then
			#echo $drive_letter
			#echo $command_output
			if [ "$check_command" == "COUNTER" ] ; then
				if [ $command_output -gt 0 ] ; then
					exit_message+=" $drive_letter:"
				fi
			else
				exit_message+=" $drive_letter:"
			fi
		fi
	done
else
	# Agent did not respond
	printf "Something went wrong: $test_command_output\n"
	exit 3
fi

if [ "$exit_message" == "Drives Detected:" ] ; then
	printf "Something went wrong: $command_output\n"
	exit 3
else
	echo $exit_message
	exit 0
fi
